/*
 * This program generates a number of "pseudo-words" with the same
 * Markov probabilities between individual characters as in the 
 * input file. The resulting words are then filtered against a simple
 * syntax checker to ensure that words that are obviously incorrect
 * are dropped "filtered out".
 *
 * This way of thinking about language has nothing to do with,
 * linguistics, a linguist would use a sound-model based in fonemics
 * (structural grammatics) or other Chomskyan structure generation
 * principles. But this is more fun. It is also simple :-)
 *
 * Written by Linus Walleij (C) under GNU GPL v2.
 */
class Pseudo {
    private Lexer mrLexer;
    private Syntax mrSyntax;
    private CharStatStore[] statstores;
    private int characters = 0;
    private char lastchar = ' ';

    public static void main(String[] args) {
	String filename;
	int words;

	if (args == null) {
	    System.err.println("Usage: java Pseudo <filename> <num>");
	    System.err.println("where <filename> is the file to analyze, <num> number of words to generate.");
	    System.exit(1);
	}
	filename = args[0];
	words = Integer.parseInt(args[1]);
	Pseudo mrPseudo = new Pseudo(filename);
	mrPseudo.runAnalysis();
	mrPseudo.calculateStats();
	System.out.println("Generated text:");
	mrPseudo.generateWords(words);
    }

    public Pseudo (String infile) {
	mrLexer = new Lexer(infile);
	mrSyntax = new SwedishSyntax();
    }

    private void runAnalysis() {
	/* Create a space for statistics */
	statstores = new CharStatStore[256];
	String word = "";
	/* Open the input stream */
	char foo = mrLexer.get();
	while (foo != '\0') {
	    characters ++;
	    addToStats(foo, lastchar);
	    lastchar = foo;
	    if (lastchar != ' ')
		word += lastchar;
	    else {
		// Send a copy of each word to the Syntax
		// analyzer for making statistics of allowed word
		// structures.
		mrSyntax.consumeWord(word);
		word = "";
	    }
	    // Uncomment to flush text to screen while analyzing
	    // System.out.print(foo);
	    
	    foo = mrLexer.get();
	}
	System.out.println(characters + " characters as indata.");
    }

    private void addToStats(char ch, char lastchar) {
	int lastval = (int) lastchar; // Character.getNumericValue(lastchar);
	if (lastval < 0 || lastval > 255)
	    return;
	if (lastval < 256) {
	    if (statstores[lastval] == null) {
		statstores[lastval] = new CharStatStore(lastchar, lastval);
		statstores[lastval].addOccurance(ch);
	    } else {
		statstores[lastval].addOccurance(ch);
	    }
	    // System.out.print(lastval);
	}
    }

    private void calculateStats() {
	for (int i = 0; i < 256; i++) {
	    if (statstores[i] != null) {
		CharStatStore store = statstores[i];
		store.calcStats(characters);
		// Uncomment to print out statistics after run
		// store.dumpStats();
	    }
	}
    }

    private void generateWords(int words) {
	Wordgenerator mrGenerator = new Wordgenerator(statstores, mrSyntax);
	mrGenerator.generateWords(words);
    }
}



